@@ -1,6 +1,6 @@
-from simtk.openmm.app import *
-from simtk.openmm import *
-from simtk.unit import *
+from openmm.app import *
+from openmm import *
+from openmm.unit import *
 from sys import stdout
 
 prmtop = AmberPrmtopFile('input.prmtop')
