@@ -9,7 +9,7 @@
  * Biological Structures at Stanford, funded under the NIH Roadmap for        *
  * Medical Research, grant U54 GM072970. See https://simtk.org.               *
  *                                                                            *
- * Portions copyright (c) 2008-2020 Stanford University and the Authors.      *
+ * Portions copyright (c) 2008-2021 Stanford University and the Authors.      *
  * Authors: Peter Eastman                                                     *
  * Contributors:                                                              *
  *                                                                            *
@@ -93,11 +93,14 @@ namespace OpenMM {
  * 
  * The "effective" parameters for a particle (those used to compute forces) are given by
  * 
- * <tt><pre>
- * charge = baseCharge + param*chargeScale
- * sigma = baseSigma + param*sigmaScale
- * epsilon = baseEpsilon + param*epsilonScale
- * </pre></tt>
+ * \verbatim embed:rst:leading-asterisk
+ * .. code-block:: cpp
+ *
+ *    charge = baseCharge + param*chargeScale
+ *    sigma = baseSigma + param*sigmaScale
+ *    epsilon = baseEpsilon + param*epsilonScale
+ *
+ * \endverbatim
  * 
  * where the "base" values are the ones specified by addParticle() and "oaram" is the current value
  * of the Context parameter.  A single Context parameter can apply offsets to multiple particles,
@@ -543,6 +546,18 @@ public:
      *                 that is specified for direct space.
      */
     void setReciprocalSpaceForceGroup(int group);
+    /**
+     * Get whether to include direct space interactions when calculating forces and energies.  This is useful if you want
+     * to completely replace the direct space calculation, typically with a CustomNonbondedForce that computes it in a
+     * nonstandard way, while still using this object for the reciprocal space calculation.
+     */
+    bool getIncludeDirectSpace() const;
+    /**
+     * Set whether to include direct space interactions when calculating forces and energies.  This is useful if you want
+     * to completely replace the direct space calculation, typically with a CustomNonbondedForce that computes it in a
+     * nonstandard way, while still using this object for the reciprocal space calculation.
+     */
+    void setIncludeDirectSpace(bool include);
     /**
      * Update the particle and exception parameters in a Context to match those stored in this Force object.  This method
      * provides an efficient method to update certain parameters in an existing Context without needing to reinitialize it.
@@ -604,7 +619,7 @@ private:
     class ExceptionOffsetInfo;
     NonbondedMethod nonbondedMethod;
     double cutoffDistance, switchingDistance, rfDielectric, ewaldErrorTol, alpha, dalpha;
-    bool useSwitchingFunction, useDispersionCorrection, exceptionsUsePeriodic;
+    bool useSwitchingFunction, useDispersionCorrection, exceptionsUsePeriodic, includeDirectSpace;
     int recipForceGroup, nx, ny, nz, dnx, dny, dnz;
     void addExclusionsToSet(const std::vector<std::set<int> >& bonded12, std::set<int>& exclusions, int baseParticle, int fromParticle, int currentLevel) const;
     int getGlobalParameterIndex(const std::string& parameter) const;
