@@ -65,6 +65,8 @@ public:
     class ReorderListener;
     class ForcePreComputation;
     class ForcePostComputation;
+    static const int ThreadBlockSize;
+    static const int TileSize;
     ComputeContext(const System& system);
     virtual ~ComputeContext();
     /**
@@ -91,9 +93,30 @@ public:
      * doing any computation when you do not know what other code has just been executing on
      * the thread.  Platforms that rely on binding contexts to threads (such as CUDA) need to
      * implement this.
+     * 
+     * @deprecated It is recommended to use pushAsCurrent() and popAsCurrent() instead, or even better to create a ContextSelector.
+     * This provides better interoperability with other libraries that use CUDA and create
+     * their own contexts.
      */
     virtual void setAsCurrent() {
     }
+    /**
+     * Set this as the current context for the calling thread, maintaining any previous context
+     * on a stack.  This should be called before doing any computation when you do not know what
+     * other code has just been executing on the thread.  It must be paired with popAsCurrent()
+     * when you are done to restore the previous context.  Alternatively, you can create a
+     * ContextSelector object to automate this for a block of code.
+     * 
+     * Platforms that rely on binding contexts to threads (such as CUDA) need to implement this.
+     */
+    virtual void pushAsCurrent() {
+    }
+    /**
+     * Restore a previous context that was replaced by pushAsCurrent().  Platforms that rely on binding
+     * contexts to threads (such as CUDA) need to implement this.
+     */
+    virtual void popAsCurrent() {
+    }
     /**
      * Get the number of contexts being used for the current simulation.
      * This is relevant when a simulation is parallelized across multiple devices.  In that case,
@@ -122,6 +145,13 @@ public:
      * @param defines            a set of preprocessor definitions (name, value) to define when compiling the program
      */
     virtual ComputeProgram compileProgram(const std::string source, const std::map<std::string, std::string>& defines=std::map<std::string, std::string>()) = 0;
+    /**
+     * Compute the largest thread block size that can be used for a kernel that requires a particular amount of
+     * shared memory per thread.
+     * 
+     * @param memory        the number of bytes of shared memory per thread
+     */
+    virtual int computeThreadBlockSize(double memory) const = 0;
     /**
      * Set all elements of an array to 0.
      */
@@ -170,13 +200,13 @@ public:
     /**
      * Get the number of integration steps that have been taken.
      */
-    int getStepCount() {
+    long long getStepCount() {
         return stepCount;
     }
     /**
      * Set the number of integration steps that have been taken.
      */
-    void setStepCount(int steps) {
+    void setStepCount(long long steps) {
         stepCount = steps;
     }
     /**
@@ -278,6 +308,10 @@ public:
     int getPaddedNumAtoms() const {
         return paddedNumAtoms;
     }
+    /**
+     * Get the number of blocks of TileSize atoms.
+     */
+    virtual int getNumAtomBlocks() const = 0;
     /**
      * Get the standard number of thread blocks to use when executing kernels.
      */
@@ -300,10 +334,14 @@ public:
     virtual ArrayInterface& getVelm() = 0;
     /**
      * On devices that do not support 64 bit atomics, this returns an array containing buffers of type real4 in which
-     * forces can be accumulated.  Do not call this if getSupports64BitGlobalAtomics() returns true.  The returned value
-     * in that case is undefined, and it may throw an exception.
+     * forces can be accumulated.  On platforms that do not use floating point force buffers, this will throw an exception.
      */
     virtual ArrayInterface& getForceBuffers() = 0;
+    /**
+     * Get the array which contains a contribution to each force represented as a real4.  On platforms that do not use
+     * floating point force buffers, this will throw an exception.
+     */
+    virtual ArrayInterface& getFloatForceBuffer() = 0;
     /**
      * Get the array which contains a contribution to each force represented as 64 bit fixed point.
      */
@@ -394,6 +432,13 @@ public:
      * Get the NonbondedUtilities for this context.
      */
     virtual NonbondedUtilities& getNonbondedUtilities() = 0;
+    /**
+     * Create a new NonbondedUtilities for use with this context.  This should be called
+     * only in unusual situations, when a Force needs its own NonbondedUtilities object
+     * separate from the standard one.  The caller is responsible for deleting the object
+     * when it is no longer needed.
+     */
+    virtual NonbondedUtilities* createNonbondedUtilities() = 0;
     /**
      * This should be called by the Integrator from its own initialize() method.
      * It ensures all contexts are fully initialized.
@@ -456,7 +501,8 @@ protected:
     void reorderAtomsImpl();
     const System& system;
     double time;
-    int numAtoms, paddedNumAtoms, stepCount, computeForceCount, stepsSinceReorder;
+    int numAtoms, paddedNumAtoms, computeForceCount, stepsSinceReorder;
+    long long stepCount;
     bool atomsWereReordered, forcesValid;
     std::vector<ComputeForceInfo*> forces;
     std::vector<Molecule> molecules;
@@ -515,7 +561,8 @@ public:
     void flush();
 private:
     std::queue<ComputeContext::WorkTask*> tasks;
-    bool waiting, finished;
+    bool waiting, finished, threwException;
+    OpenMMException stashedException;
     pthread_mutex_t queueLock;
     pthread_cond_t waitForTaskCondition, queueEmptyCondition;
     pthread_t thread;
