@@ -25,12 +25,6 @@
  * -------------------------------------------------------------------------- */
 
 #include "AmoebaReferenceKernels.h"
-#include "AmoebaReferenceBondForce.h"
-#include "AmoebaReferenceAngleForce.h"
-#include "AmoebaReferenceInPlaneAngleForce.h"
-#include "AmoebaReferencePiTorsionForce.h"
-#include "AmoebaReferenceStretchBendForce.h"
-#include "AmoebaReferenceOutOfPlaneBendForce.h"
 #include "AmoebaReferenceTorsionTorsionForce.h"
 #include "AmoebaReferenceWcaDispersionForce.h"
 #include "AmoebaReferenceGeneralizedKirkwoodForce.h"
@@ -82,351 +76,6 @@ static Vec3* extractBoxVectors(ContextImpl& context) {
 
 // ***************************************************************************
 
-ReferenceCalcAmoebaBondForceKernel::ReferenceCalcAmoebaBondForceKernel(const std::string& name, const Platform& platform, const System& system) :
-                CalcAmoebaBondForceKernel(name, platform), system(system) {
-}
-
-ReferenceCalcAmoebaBondForceKernel::~ReferenceCalcAmoebaBondForceKernel() {
-}
-
-void ReferenceCalcAmoebaBondForceKernel::initialize(const System& system, const AmoebaBondForce& force) {
-
-    numBonds = force.getNumBonds();
-    for (int ii = 0; ii < numBonds; ii++) {
-
-        int particle1Index, particle2Index;
-        double lengthValue, kValue;
-        force.getBondParameters(ii, particle1Index, particle2Index, lengthValue, kValue);
-
-        particle1.push_back(particle1Index); 
-        particle2.push_back(particle2Index); 
-        length.push_back(static_cast<double>(lengthValue));
-        kQuadratic.push_back(kValue);
-    } 
-    globalBondCubic   = force.getAmoebaGlobalBondCubic();
-    globalBondQuartic = force.getAmoebaGlobalBondQuartic();
-    usePeriodic = force.usesPeriodicBoundaryConditions();
-}
-
-double ReferenceCalcAmoebaBondForceKernel::execute(ContextImpl& context, bool includeForces, bool includeEnergy) {
-    vector<Vec3>& posData = extractPositions(context);
-    vector<Vec3>& forceData = extractForces(context);
-    AmoebaReferenceBondForce amoebaReferenceBondForce;
-    if (usePeriodic)
-        amoebaReferenceBondForce.setPeriodic(extractBoxVectors(context));
-    double energy = amoebaReferenceBondForce.calculateForceAndEnergy(numBonds, posData, particle1, particle2, length, kQuadratic,
-                                                                     globalBondCubic, globalBondQuartic,
-                                                                     forceData);
-    return static_cast<double>(energy);
-}
-
-void ReferenceCalcAmoebaBondForceKernel::copyParametersToContext(ContextImpl& context, const AmoebaBondForce& force) {
-    if (numBonds != force.getNumBonds())
-        throw OpenMMException("updateParametersInContext: The number of bonds has changed");
-
-    // Record the values.
-
-    for (int i = 0; i < numBonds; ++i) {
-        int particle1Index, particle2Index;
-        double lengthValue, kValue;
-        force.getBondParameters(i, particle1Index, particle2Index, lengthValue, kValue);
-        if (particle1Index != particle1[i] || particle2Index != particle2[i])
-            throw OpenMMException("updateParametersInContext: The set of particles in a bond has changed");
-        length[i] = lengthValue;
-        kQuadratic[i] = kValue;
-    }
-}
-
-// ***************************************************************************
-
-ReferenceCalcAmoebaAngleForceKernel::ReferenceCalcAmoebaAngleForceKernel(const std::string& name, const Platform& platform, const System& system) :
-            CalcAmoebaAngleForceKernel(name, platform), system(system) {
-}
-
-ReferenceCalcAmoebaAngleForceKernel::~ReferenceCalcAmoebaAngleForceKernel() {
-}
-
-void ReferenceCalcAmoebaAngleForceKernel::initialize(const System& system, const AmoebaAngleForce& force) {
-
-    numAngles = force.getNumAngles();
-
-    for (int ii = 0; ii < numAngles; ii++) {
-        int particle1Index, particle2Index, particle3Index;
-        double angleValue, k;
-        force.getAngleParameters(ii, particle1Index, particle2Index, particle3Index, angleValue, k);
-        particle1.push_back(particle1Index); 
-        particle2.push_back(particle2Index); 
-        particle3.push_back(particle3Index); 
-        angle.push_back(angleValue);
-        kQuadratic.push_back(k);
-    }
-    globalAngleCubic    = force.getAmoebaGlobalAngleCubic();
-    globalAngleQuartic  = force.getAmoebaGlobalAngleQuartic();
-    globalAnglePentic   = force.getAmoebaGlobalAnglePentic();
-    globalAngleSextic   = force.getAmoebaGlobalAngleSextic();
-    usePeriodic = force.usesPeriodicBoundaryConditions();
-}
-
-double ReferenceCalcAmoebaAngleForceKernel::execute(ContextImpl& context, bool includeForces, bool includeEnergy) {
-    vector<Vec3>& posData = extractPositions(context);
-    vector<Vec3>& forceData = extractForces(context);
-    AmoebaReferenceAngleForce amoebaReferenceAngleForce;
-    if (usePeriodic)
-        amoebaReferenceAngleForce.setPeriodic(extractBoxVectors(context));
-    double energy = amoebaReferenceAngleForce.calculateForceAndEnergy(numAngles, 
-                                       posData, particle1, particle2, particle3, angle, kQuadratic, globalAngleCubic, globalAngleQuartic, globalAnglePentic, globalAngleSextic, forceData);
-    return static_cast<double>(energy);
-}
-
-void ReferenceCalcAmoebaAngleForceKernel::copyParametersToContext(ContextImpl& context, const AmoebaAngleForce& force) {
-    if (numAngles != force.getNumAngles())
-        throw OpenMMException("updateParametersInContext: The number of angles has changed");
-
-    // Record the values.
-
-    for (int i = 0; i < numAngles; ++i) {
-        int particle1Index, particle2Index, particle3Index;
-        double angleValue, k;
-        force.getAngleParameters(i, particle1Index, particle2Index, particle3Index, angleValue, k);
-        if (particle1Index != particle1[i] || particle2Index != particle2[i] || particle3Index != particle3[i])
-            throw OpenMMException("updateParametersInContext: The set of particles in an angle has changed");
-        angle[i] = angleValue;
-        kQuadratic[i] = k;
-    }
-}
-
-ReferenceCalcAmoebaInPlaneAngleForceKernel::ReferenceCalcAmoebaInPlaneAngleForceKernel(const std::string& name, const Platform& platform, const System& system) :
-          CalcAmoebaInPlaneAngleForceKernel(name, platform), system(system) {
-}
-
-ReferenceCalcAmoebaInPlaneAngleForceKernel::~ReferenceCalcAmoebaInPlaneAngleForceKernel() {
-}
-
-void ReferenceCalcAmoebaInPlaneAngleForceKernel::initialize(const System& system, const AmoebaInPlaneAngleForce& force) {
-
-    numAngles = force.getNumAngles();
-    for (int ii = 0; ii < numAngles; ii++) {
-        int particle1Index, particle2Index, particle3Index, particle4Index;
-        double angleValue, k;
-        force.getAngleParameters(ii, particle1Index, particle2Index, particle3Index, particle4Index, angleValue, k);
-        particle1.push_back(particle1Index); 
-        particle2.push_back(particle2Index); 
-        particle3.push_back(particle3Index); 
-        particle4.push_back(particle4Index); 
-        angle.push_back(angleValue);
-        kQuadratic.push_back(k);
-    }
-    globalInPlaneAngleCubic    = force.getAmoebaGlobalInPlaneAngleCubic();
-    globalInPlaneAngleQuartic  = force.getAmoebaGlobalInPlaneAngleQuartic();
-    globalInPlaneAnglePentic   = force.getAmoebaGlobalInPlaneAnglePentic();
-    globalInPlaneAngleSextic   = force.getAmoebaGlobalInPlaneAngleSextic();
-    usePeriodic = force.usesPeriodicBoundaryConditions();
-}
-
-double ReferenceCalcAmoebaInPlaneAngleForceKernel::execute(ContextImpl& context, bool includeForces, bool includeEnergy) {
-
-    vector<Vec3>& posData = extractPositions(context);
-    vector<Vec3>& forceData = extractForces(context);
-    AmoebaReferenceInPlaneAngleForce amoebaReferenceInPlaneAngleForce;
-    if (usePeriodic)
-        amoebaReferenceInPlaneAngleForce.setPeriodic(extractBoxVectors(context));
-    double energy = amoebaReferenceInPlaneAngleForce.calculateForceAndEnergy(numAngles, posData, particle1, particle2, particle3, particle4, 
-                                                                             angle, kQuadratic, globalInPlaneAngleCubic, globalInPlaneAngleQuartic,
-                                                                             globalInPlaneAnglePentic, globalInPlaneAngleSextic, forceData);
-    return static_cast<double>(energy);
-}
-
-void ReferenceCalcAmoebaInPlaneAngleForceKernel::copyParametersToContext(ContextImpl& context, const AmoebaInPlaneAngleForce& force) {
-    if (numAngles != force.getNumAngles())
-        throw OpenMMException("updateParametersInContext: The number of angles has changed");
-
-    // Record the values.
-
-    for (int i = 0; i < numAngles; ++i) {
-        int particle1Index, particle2Index, particle3Index, particle4Index;
-        double angleValue, k;
-        force.getAngleParameters(i, particle1Index, particle2Index, particle3Index, particle4Index, angleValue, k);
-        if (particle1Index != particle1[i] || particle2Index != particle2[i] || particle3Index != particle3[i] || particle4Index != particle4[i])
-            throw OpenMMException("updateParametersInContext: The set of particles in an angle has changed");
-        angle[i] = angleValue;
-        kQuadratic[i] = k;
-    }
-}
-
-ReferenceCalcAmoebaPiTorsionForceKernel::ReferenceCalcAmoebaPiTorsionForceKernel(const std::string& name, const Platform& platform, const System& system) :
-         CalcAmoebaPiTorsionForceKernel(name, platform), system(system) {
-}
-
-ReferenceCalcAmoebaPiTorsionForceKernel::~ReferenceCalcAmoebaPiTorsionForceKernel() {
-}
-
-void ReferenceCalcAmoebaPiTorsionForceKernel::initialize(const System& system, const AmoebaPiTorsionForce& force) {
-
-    numPiTorsions                     = force.getNumPiTorsions();
-    for (int ii = 0; ii < numPiTorsions; ii++) {
-
-        int particle1Index, particle2Index, particle3Index, particle4Index, particle5Index, particle6Index;
-        double kTorsionParameter;
-        force.getPiTorsionParameters(ii, particle1Index, particle2Index, particle3Index, particle4Index, particle5Index, particle6Index, kTorsionParameter);
-        particle1.push_back(particle1Index); 
-        particle2.push_back(particle2Index); 
-        particle3.push_back(particle3Index); 
-        particle4.push_back(particle4Index); 
-        particle5.push_back(particle5Index); 
-        particle6.push_back(particle6Index); 
-        kTorsion.push_back(kTorsionParameter);
-    }
-    usePeriodic = force.usesPeriodicBoundaryConditions();
-}
-
-double ReferenceCalcAmoebaPiTorsionForceKernel::execute(ContextImpl& context, bool includeForces, bool includeEnergy) {
-    vector<Vec3>& posData = extractPositions(context);
-    vector<Vec3>& forceData = extractForces(context);
-    AmoebaReferencePiTorsionForce amoebaReferencePiTorsionForce;
-    if (usePeriodic)
-        amoebaReferencePiTorsionForce.setPeriodic(extractBoxVectors(context));
-    double energy = amoebaReferencePiTorsionForce.calculateForceAndEnergy(numPiTorsions, posData, particle1, particle2,
-                                                                                    particle3, particle4, particle5, particle6,
-                                                                                    kTorsion, forceData);
-    return static_cast<double>(energy);
-}
-
-void ReferenceCalcAmoebaPiTorsionForceKernel::copyParametersToContext(ContextImpl& context, const AmoebaPiTorsionForce& force) {
-    if (numPiTorsions != force.getNumPiTorsions())
-        throw OpenMMException("updateParametersInContext: The number of torsions has changed");
-
-    // Record the values.
-
-    for (int i = 0; i < numPiTorsions; ++i) {
-        int particle1Index, particle2Index, particle3Index, particle4Index, particle5Index, particle6Index;
-        double kTorsionParameter;
-        force.getPiTorsionParameters(i, particle1Index, particle2Index, particle3Index, particle4Index, particle5Index, particle6Index, kTorsionParameter);
-        if (particle1Index != particle1[i] || particle2Index != particle2[i] || particle3Index != particle3[i] ||
-            particle4Index != particle4[i] || particle5Index != particle5[i] || particle6Index != particle6[i])
-            throw OpenMMException("updateParametersInContext: The set of particles in a torsion has changed");
-        kTorsion[i] = kTorsionParameter;
-    }
-}
-
-ReferenceCalcAmoebaStretchBendForceKernel::ReferenceCalcAmoebaStretchBendForceKernel(const std::string& name, const Platform& platform, const System& system) :
-                   CalcAmoebaStretchBendForceKernel(name, platform), system(system) {
-}
-
-ReferenceCalcAmoebaStretchBendForceKernel::~ReferenceCalcAmoebaStretchBendForceKernel() {
-}
-
-void ReferenceCalcAmoebaStretchBendForceKernel::initialize(const System& system, const AmoebaStretchBendForce& force) {
-
-    numStretchBends = force.getNumStretchBends();
-    for (int ii = 0; ii < numStretchBends; ii++) {
-        int particle1Index, particle2Index, particle3Index;
-        double lengthAB, lengthCB, angle, k1, k2;
-        force.getStretchBendParameters(ii, particle1Index, particle2Index, particle3Index, lengthAB, lengthCB, angle, k1, k2);
-        particle1.push_back(particle1Index); 
-        particle2.push_back(particle2Index); 
-        particle3.push_back(particle3Index); 
-        lengthABParameters.push_back(lengthAB);
-        lengthCBParameters.push_back(lengthCB);
-        angleParameters.push_back(angle);
-        k1Parameters.push_back(k1);
-        k2Parameters.push_back(k2);
-    }
-    usePeriodic = force.usesPeriodicBoundaryConditions();
-}
-
-double ReferenceCalcAmoebaStretchBendForceKernel::execute(ContextImpl& context, bool includeForces, bool includeEnergy) {
-    vector<Vec3>& posData = extractPositions(context);
-    vector<Vec3>& forceData = extractForces(context);
-    AmoebaReferenceStretchBendForce amoebaReferenceStretchBendForce;
-    if (usePeriodic)
-        amoebaReferenceStretchBendForce.setPeriodic(extractBoxVectors(context));
-    double energy = amoebaReferenceStretchBendForce.calculateForceAndEnergy(numStretchBends, posData, particle1, particle2, particle3,
-                                                                                      lengthABParameters, lengthCBParameters, angleParameters, k1Parameters,
-                                                                                      k2Parameters, forceData);
-    return static_cast<double>(energy);
-}
-
-void ReferenceCalcAmoebaStretchBendForceKernel::copyParametersToContext(ContextImpl& context, const AmoebaStretchBendForce& force) {
-    if (numStretchBends != force.getNumStretchBends())
-        throw OpenMMException("updateParametersInContext: The number of stretch-bends has changed");
-
-    // Record the values.
-
-    for (int i = 0; i < numStretchBends; ++i) {
-        int particle1Index, particle2Index, particle3Index;
-        double lengthAB, lengthCB, angle, k1, k2;
-        force.getStretchBendParameters(i, particle1Index, particle2Index, particle3Index, lengthAB, lengthCB, angle, k1, k2);
-        if (particle1Index != particle1[i] || particle2Index != particle2[i] || particle3Index != particle3[i])
-            throw OpenMMException("updateParametersInContext: The set of particles in a stretch-bend has changed");
-        lengthABParameters[i] = lengthAB;
-        lengthCBParameters[i] = lengthCB;
-        angleParameters[i] = angle;
-        k1Parameters[i] = k1;
-        k2Parameters[i] = k2;
-    }
-}
-
-ReferenceCalcAmoebaOutOfPlaneBendForceKernel::ReferenceCalcAmoebaOutOfPlaneBendForceKernel(const std::string& name, const Platform& platform, const System& system) :
-          CalcAmoebaOutOfPlaneBendForceKernel(name, platform), system(system) {
-}
-
-ReferenceCalcAmoebaOutOfPlaneBendForceKernel::~ReferenceCalcAmoebaOutOfPlaneBendForceKernel() {
-}
-
-void ReferenceCalcAmoebaOutOfPlaneBendForceKernel::initialize(const System& system, const AmoebaOutOfPlaneBendForce& force) {
-
-    numOutOfPlaneBends = force.getNumOutOfPlaneBends();
-    for (int ii = 0; ii < numOutOfPlaneBends; ii++) {
-
-        int particle1Index, particle2Index, particle3Index, particle4Index;
-        double k;
-
-        force.getOutOfPlaneBendParameters(ii, particle1Index, particle2Index, particle3Index, particle4Index, k);
-        particle1.push_back(particle1Index); 
-        particle2.push_back(particle2Index); 
-        particle3.push_back(particle3Index); 
-        particle4.push_back(particle4Index); 
-        kParameters.push_back(k);
-    }
-    globalOutOfPlaneBendAngleCubic   = force.getAmoebaGlobalOutOfPlaneBendCubic();
-    globalOutOfPlaneBendAngleQuartic = force.getAmoebaGlobalOutOfPlaneBendQuartic();
-    globalOutOfPlaneBendAnglePentic  = force.getAmoebaGlobalOutOfPlaneBendPentic();
-    globalOutOfPlaneBendAngleSextic  = force.getAmoebaGlobalOutOfPlaneBendSextic();
-    usePeriodic = force.usesPeriodicBoundaryConditions();
-}
-
-double ReferenceCalcAmoebaOutOfPlaneBendForceKernel::execute(ContextImpl& context, bool includeForces, bool includeEnergy) {
-    vector<Vec3>& posData = extractPositions(context);
-    vector<Vec3>& forceData = extractForces(context);
-    AmoebaReferenceOutOfPlaneBendForce amoebaReferenceOutOfPlaneBendForce;
-    if (usePeriodic)
-        amoebaReferenceOutOfPlaneBendForce.setPeriodic(extractBoxVectors(context));
-    double energy = amoebaReferenceOutOfPlaneBendForce.calculateForceAndEnergy(numOutOfPlaneBends, posData,
-                                                                               particle1, particle2, particle3, particle4,
-                                                                               kParameters, 
-                                                                               globalOutOfPlaneBendAngleCubic,
-                                                                               globalOutOfPlaneBendAngleQuartic,
-                                                                               globalOutOfPlaneBendAnglePentic,
-                                                                               globalOutOfPlaneBendAngleSextic, forceData); 
-    return static_cast<double>(energy);
-}
-
-void ReferenceCalcAmoebaOutOfPlaneBendForceKernel::copyParametersToContext(ContextImpl& context, const AmoebaOutOfPlaneBendForce& force) {
-    if (numOutOfPlaneBends != force.getNumOutOfPlaneBends())
-        throw OpenMMException("updateParametersInContext: The number of out-of-plane bends has changed");
-
-    // Record the values.
-
-    for (int i = 0; i < numOutOfPlaneBends; ++i) {
-        int particle1Index, particle2Index, particle3Index, particle4Index;
-        double k;
-        force.getOutOfPlaneBendParameters(i, particle1Index, particle2Index, particle3Index, particle4Index, k);
-        if (particle1Index != particle1[i] || particle2Index != particle2[i] || particle3Index != particle3[i] || particle4Index != particle4[i])
-            throw OpenMMException("updateParametersInContext: The set of particles in an out-of-plane bend has changed");
-        kParameters[i] = k;
-    }
-}
-
 ReferenceCalcAmoebaTorsionTorsionForceKernel::ReferenceCalcAmoebaTorsionTorsionForceKernel(const std::string& name, const Platform& platform, const System& system) :
                 CalcAmoebaTorsionTorsionForceKernel(name, platform), system(system) {
 }
